import __AS3__.vec.Vector;

import com.degrafa.GeometryGroup;
import com.degrafa.core.collections.SegmentsCollection;
import com.degrafa.geometry.AdvancedCubicBezier;
import com.degrafa.geometry.Path;
import com.degrafa.geometry.segment.CubicBezierTo;
import com.degrafa.geometry.segment.ISegment;
import com.degrafa.geometry.segment.MoveTo;
import com.degrafa.paint.SolidStroke;

import de.popforge.audio.output.Audio;
import de.popforge.audio.output.Sample;
import de.popforge.format.wav.WavEncoder;

import flash.events.Event;
import flash.events.MouseEvent;
import flash.geom.Matrix3D;
import flash.geom.PerspectiveProjection;
import flash.geom.Point;
import flash.geom.Vector3D;

import mx.controls.Alert;
import mx.events.PropertyChangeEvent;

import org.casalib.math.Percent;
import org.casalib.util.NumberUtil;

private const TOTAL_POINT_ROW:uint = 40;
private const POINT_ROW_LENGTH:uint = 10;
private const POINT_MAX_HEIGHT:Number = 30;
private const POINT_MIN_HEIGHT:Number = -30;
private const CURVE_WIDTH:Number = 400;
private const CYLINDER_RADIUS:Number = 50;
private const IPOINT_UP_COLOR:uint = 0x99FF99;
private const IPOINT_DOWN_COLOR:uint = 0x99CC99;
private const IPOINT_PRESS_COLOR:uint = 0x99DD99;
private const IPOINT_DEFAULT_COLOR:uint = 0x555555;
private var BUFFER_LENGTH:uint = 8 * 3 * 1024; //min:8*2048 max:8*8192

[Bindable]
private var crvHandleSize:Number = 20;
[Bindable]
private var crvStroke:SolidStroke = new SolidStroke(0xDDDDDD);
private var points:Vector.<Vector.<Point>>;
private var paths:Vector.<Path>;
private var interactivePts:Vector.<InteractivePoint>;
private var draggingPt:InteractivePoint;
private var currentEditingCurveIndex:int;
private var currentSoundCurve:Path;
private var cylinderRotationPerFrame:Number = 1;
private var soundBytes:ByteArray;
private var tempSoundBytes:ByteArray;
private var tempBezier:AdvancedCubicBezier = new AdvancedCubicBezier();
[Bindable]
private var baseFrequency:Number = 220;
private var sound:Sound;
private var soundChannel:SoundChannel;
private var isPullingPt:Boolean = false;
private var lastFrameRate:Number = 0;
private var lastTimer:Number = 0;
[Bindable]
public var enableRotateCylinder:Boolean = true;

private var save_totalNumOfInterpolations:uint;
private var save_numOfInterpolations:uint;
private var save_sampleArray:Array = [];
private var save_interpolationsPerFrame:uint = 20;
private var save_cylinderRotationX:Number = 0;
private var save_adjustStep:uint = 2;

//private var debugPt:InteractivePoint = new InteractivePoint();

private var i:uint, j:uint; //some counters for for loops

private function init():void {
	points = new Vector.<Vector.<Point>>(TOTAL_POINT_ROW);
	paths = new Vector.<Path>(TOTAL_POINT_ROW);

	var perspectiveProjection:PerspectiveProjection = new PerspectiveProjection();
	var subSurface:Surface;
	var geoGroup:GeometryGroup;
	perspectiveProjection.projectionCenter = new Point(0, 0);
	curveSurface.transform.perspectiveProjection = perspectiveProjection;
	for (var i:uint = 0; i < TOTAL_POINT_ROW; ++i) {
		points[i] = new Vector.<Point>(POINT_ROW_LENGTH);
		for (var j:uint = 0; j < POINT_ROW_LENGTH; ++j) {
			points[i][j] = new Point(-CURVE_WIDTH * 0.5 + j * (CURVE_WIDTH / POINT_ROW_LENGTH) + 20, -CYLINDER_RADIUS + Math.max(POINT_MIN_HEIGHT, 0));
		}

		//construct the curve and add it to the stage
		paths[i] = makeCurve(points[i]);
		subSurface = new Surface();
		geoGroup = new GeometryGroup();
		curveSurface.addChild(subSurface);
		subSurface.addChild(geoGroup);
		geoGroup.geometryCollection.addItem(paths[i]);
		subSurface.rotationX = 360 / TOTAL_POINT_ROW * i;
		subSurface.transform.perspectiveProjection = perspectiveProjection;
	}
	currentEditingCurveIndex = 0;

	currentSoundCurve = makeCurve(points[currentEditingCurveIndex]);
	currentSoundCurve.stroke = new SolidStroke(0xFFFFFF, 1, 5);
	subSurface = new Surface();
	subSurface.mouseEnabled = false;
	geoGroup = new GeometryGroup();
	pointControlCanvas.addChild(subSurface);
	subSurface.addChild(geoGroup);
	geoGroup.geometryCollection.addItem(currentSoundCurve);

	interactivePts = new Vector.<InteractivePoint>(POINT_ROW_LENGTH);
	for (i = 0; i < POINT_ROW_LENGTH; ++i) {
		var ipt:InteractivePoint = new InteractivePoint();
		ipt.label = i.toString();
		ipt.pointLabel = ((i + 1) % 10).toString();
		ipt.width = 100;
		ipt.height = 20;
		ipt.radius = 5;
		ipt.color = IPOINT_DEFAULT_COLOR;
		ipt.move(points[currentEditingCurveIndex][i].x, points[currentEditingCurveIndex][i].y);
		ipt.restrict = new Rectangle(ipt.x, ipt.y - POINT_MAX_HEIGHT, 0, POINT_MAX_HEIGHT - POINT_MIN_HEIGHT);
		ipt.addEventListener(PropertyChangeEvent.PROPERTY_CHANGE, onPropertyChanged);
		pointControlCanvas.addChild(ipt);
		interactivePts[i] = ipt;
	}

	//first and last point is fixed
	interactivePts[0].removeEventListener(PropertyChangeEvent.PROPERTY_CHANGE, onPropertyChanged);
	interactivePts[0].mouseEnabled = false;
	interactivePts[0].pointLabel = "fixed";
	--i;
	interactivePts[i].removeEventListener(PropertyChangeEvent.PROPERTY_CHANGE, onPropertyChanged);
	interactivePts[i].mouseEnabled = false;
	interactivePts[i].pointLabel = "fixed";

	tempSoundBytes = new ByteArray();
	soundBytes = new ByteArray();
	updateBaseFrequency();

	addEventListener(Event.ENTER_FRAME, onEnterFrame);
	systemManager.stage.addEventListener(KeyboardEvent.KEY_DOWN, onKeyDown);
	systemManager.stage.addEventListener(KeyboardEvent.KEY_UP, onKeyUp);

	sound = new Sound();
	sound.addEventListener(SampleDataEvent.SAMPLE_DATA, addSoundBytesToSound, false, 0, true);
	soundChannel = sound.play();
	soundChannel.addEventListener(Event.SOUND_COMPLETE, replaySound);

	exportWindow.saveBtn.addEventListener(MouseEvent.CLICK, onSaveBtnClicked);

	addEventListener(Event.ENTER_FRAME, checkFrameRate, false, 10, true);
/*debugPt.label = debugPt.pointLabel = "debug";
   debugPt.width = 100;
   debugPt.height = 20;
   debugPt.radius = 5;
   debugPt.restrict = new Rectangle(-systemManager.stage.stageWidth*0.5,-systemManager.stage.stageHeight*0.5,systemManager.stage.stageWidth,systemManager.stage.stageHeight);
 pointControlCanvas.addChild(debugPt);*/
}

private function checkFrameRate(evt:Event):void {
	lastFrameRate = lastFrameRate * 0.5 + 500 / (getTimer() - lastTimer);
	lastTimer = getTimer();
	//trace("lastFrameRate", lastFrameRate);
}

private function replaySound(evt:Event):void {
	if (!exportWindow.visible)
		soundChannel = sound.play();
}

private function initExportWindow():void {
	exportWindow.saveBtn.addEventListener(MouseEvent.CLICK, onSaveBtnClicked);
}

private function save_onEnterFrame(evt:Event):void {
	//trace(save_numOfInterpolations,save_sampleArray.length);
	for (var i:uint = 0; i < save_interpolationsPerFrame; ++i) {
		onEnterFrame();
		soundBytes.position = 0;
		while (soundBytes.bytesAvailable) {
			var sample:Sample = new Sample(soundBytes.readFloat(), soundBytes.readFloat());
			save_sampleArray.push(sample);
		}
		if (--save_numOfInterpolations <= 0) {
			removeEventListener(Event.ENTER_FRAME, save_onEnterFrame);
			Alert.show("Click OK to save file on hard disk", "Finishing export", 4, null, finishSaving);
			return;
		}
	}
	if (lastFrameRate * 1.2 < systemManager.stage.frameRate && save_interpolationsPerFrame - save_adjustStep >= 1)
		save_interpolationsPerFrame -= save_adjustStep;
	else if (lastFrameRate * 1.2 > systemManager.stage.frameRate)
		save_interpolationsPerFrame += save_adjustStep;

	//trace("save_interpolationsPerFrame", save_interpolationsPerFrame);
	exportWindow.processBar.setProgress(save_totalNumOfInterpolations - save_numOfInterpolations, save_totalNumOfInterpolations);
}

private function onSaveBtnClicked(evt:MouseEvent):void {
	pointControlCanvas.mouseEnabled = false;
	pointControlCanvas.mouseChildren = false;
	exportWindow.saveBtn.enabled = false;
	controlBox.enabled = false;
	enableRotateCylinder = false;
	removeEventListener(Event.ENTER_FRAME, onEnterFrame);
	soundChannel.removeEventListener(Event.SOUND_COMPLETE, replaySound);
	var samplesPerCurve:uint = tempSoundBytes.length * 0.125; // tempSoundBytes.length/8
	if (exportWindow.exportType.selectedLabel == "whole cylinder") {
		var samplesNeeded:Number = exportWindow.durationOfCylinderLoop.value * 44100;
		save_numOfInterpolations = samplesNeeded / samplesPerCurve;
		cylinderRotationPerFrame = 360 / save_numOfInterpolations;
		save_numOfInterpolations *= exportWindow.numOfCylinderLoop.value;
		trace("durationOfCylinderLoop", exportWindow.durationOfCylinderLoop.value);
		trace("samplesNeeded", samplesNeeded);
		trace("save_numOfInterpolations", save_numOfInterpolations);
		save_totalNumOfInterpolations = save_numOfInterpolations;
		addEventListener(Event.ENTER_FRAME, save_onEnterFrame);
	} else if (exportWindow.exportType.selectedLabel == "current curve only") {
		soundBytes.position = 0;
		while (soundBytes.bytesAvailable) {
			var sample:Sample = new Sample(soundBytes.readFloat(), soundBytes.readFloat());
			save_sampleArray.push(sample);
		}
		
		var tempArray:Array = save_sampleArray.concat();
		for (var n:uint = 0 ; n < exportWindow.numOfWaveLoop.value ; ++n){
			save_sampleArray = save_sampleArray.concat(tempArray);
		}
		tempArray.length = 0;
		callLater(Alert.show,["Click OK to save file on hard disk", "Finishing export", 4, null, finishSaving]);
	}
}

private function finishSaving(evt:Event = null):void {
	var fr:FileReference = new FileReference();
	fr.save(WavEncoder.encode(save_sampleArray, Audio.STEREO, Audio.BIT16, Audio.RATE44100), new Date().toDateString() + ".wav");
	save_sampleArray.length = 0;
	cylinderRotationPerFrame = 1;
	enableRotateCylinder = true;
	addEventListener(Event.ENTER_FRAME, onEnterFrame);
	sound.addEventListener(SampleDataEvent.SAMPLE_DATA, addSoundBytesToSound, false, 0, true);
	soundChannel = sound.play();
	soundChannel.addEventListener(Event.SOUND_COMPLETE, replaySound);
	exportWindow.saveBtn.enabled = true;
	controlBox.enabled = true;
	pointControlCanvas.mouseEnabled = true;
	pointControlCanvas.mouseChildren = true;
}

private function updateBaseFrequency():void {
	tempSoundBytes.length = 44100 * 4 * 2 / baseFrequency; //sample rate * size of float * channels / frequency
	while (tempSoundBytes.length % 8 != 0)
		tempSoundBytes.length++;
	soundBytes.length = tempSoundBytes.length;
	soundBytes.writeBytes(tempSoundBytes, 0, tempSoundBytes.length);
}

private function addSoundBytesToSound(event:SampleDataEvent):void {
	if (save_sampleArray.length == 0) {
		var bytes:ByteArray = new ByteArray();
		while (bytes.length + soundBytes.length < BUFFER_LENGTH) {
			soundBytes.position = 0;
			bytes.writeBytes(soundBytes, 0, soundBytes.length);
		}
		event.data.writeBytes(bytes, 0, bytes.length);
		bytes.clear();
			//trace("addSoundBytesToSound",bytes.length)
	}
}

private function updateCurrentSoundCurve():void {
	var segs:SegmentsCollection = currentSoundCurve.segmentCollection;
	var crv:CubicBezierTo;
	(segs.getItemAt(0) as MoveTo).x = interactivePts[0].x;
	(segs.getItemAt(0) as MoveTo).y = interactivePts[0].y;

	for (i = 1; i < interactivePts.length; ++i) {
		//curve handle is horizontally placed
		crv = segs.getItemAt(i) as CubicBezierTo;
		crv.cx = interactivePts[i - 1].x + crvHandleSize;
		crv.cy = interactivePts[i - 1].y;
		crv.cx1 = interactivePts[i].x - crvHandleSize;
		crv.cy1 = interactivePts[i].y;
		crv.x = interactivePts[i].x;
		crv.y = interactivePts[i].y;
	}
}

private function onKeyDown(evt:KeyboardEvent):void {
	if (pointControlCanvas.mouseEnabled) {
		var key:String = String.fromCharCode(evt.keyCode);
		if (evt.keyCode == Keyboard.SHIFT) {
			isPullingPt = true;
		} else {
			try {
				var num:uint = Number(key);
				if (num == 0)
					num = 10;
				if (num == 1 || num == POINT_ROW_LENGTH)
					return;
				--num;
				if (!isPullingPt) {
					interactivePts[num].color = IPOINT_DOWN_COLOR;
				} else {
					interactivePts[num].color = IPOINT_UP_COLOR;
				}
			} catch (err:Error) {
				return;
			}
		}
	}

}

private function onKeyUp(evt:KeyboardEvent):void {
	if (pointControlCanvas.mouseEnabled) {
		var key:String = String.fromCharCode(evt.keyCode);
		if (evt.keyCode == Keyboard.SHIFT) {
			isPullingPt = false;
		} else {
			try {
				var num:uint = Number(key);
				if (num == 0)
					num = 10;
				interactivePts[--num].color = IPOINT_DEFAULT_COLOR;
			} catch (err:Error) {
				return;
			}
		}
	}
}


private function onPropertyChanged(evt:PropertyChangeEvent):void {
	switch (evt.property) {
		case InteractivePoint.MOUSE_DOWN:
			draggingPt = evt.target as InteractivePoint;
			draggingPt.color = IPOINT_PRESS_COLOR;
			if (Number(draggingPt.label) == POINT_ROW_LENGTH - 1) {
				interactivePts[0].color = IPOINT_PRESS_COLOR;
			}
			if (Number(draggingPt.label) == 0) {
				interactivePts[POINT_ROW_LENGTH - 1].color = IPOINT_PRESS_COLOR;
			}
			addEventListener(Event.ENTER_FRAME, onPointMove);
			break;
		case InteractivePoint.MOUSE_UP:
			removeEventListener(Event.ENTER_FRAME, onPointMove);
			if (draggingPt) {
				if (Number(draggingPt.label) == POINT_ROW_LENGTH - 1) {
					interactivePts[0].color = IPOINT_DEFAULT_COLOR;
				}
				if (Number(draggingPt.label) == 0) {
					interactivePts[POINT_ROW_LENGTH - 1].color = IPOINT_DEFAULT_COLOR;
				}
				draggingPt.color = IPOINT_DEFAULT_COLOR;
				draggingPt = null;
			}
			break;
	}
}

private function onPointMove(evt:Event = null):void {
	if (draggingPt) {
		if (Number(draggingPt.label) == POINT_ROW_LENGTH - 1) {
			interactivePts[0].y = draggingPt.y;
		}
		if (Number(draggingPt.label) == 0) {
			interactivePts[POINT_ROW_LENGTH - 1].y = draggingPt.y;
		}
	}
}


private function makeCurve(pts:Vector.<Point>, controlPts:Vector.<Point> = null):Path {
	var path:Path = new Path(), segs:SegmentsCollection = path.segmentCollection;
	var crv:CubicBezierTo;
	segs.addItem(new MoveTo(pts[0].x, pts[0].y));
	if (controlPts == null) {
		for (var i:uint = 1; i < pts.length; ++i) {
			//curve handle is horizontally placed
			crv = new CubicBezierTo(pts[i - 1].x + crvHandleSize, pts[i - 1].y, pts[i].x - crvHandleSize, pts[i].y, pts[i].x, pts[i].y);
			segs.addItem(crv);
		}
	}
	path.stroke = crvStroke;
	return path;
}

private function onEnterFrame(evt:Event = null):void {
	var num:Number;

	curveSurface.suppressEventProcessing = true;

	//rotate cylinder
	if (enableRotateCylinder || curveSurface.rotationX % (360 / TOTAL_POINT_ROW) != 0) {
		if (save_sampleArray.length == 0) {
			curveSurface.rotationX = NumberUtil.loopIndex(curveSurface.rotationX += cylinderRotationPerFrame, 360);
		}

		else {
			save_cylinderRotationX += cylinderRotationPerFrame
			if (save_cylinderRotationX > 360) {
				save_cylinderRotationX - 360;
			}
		}
	}

	if (save_sampleArray.length == 0)
		num = TOTAL_POINT_ROW - curveSurface.rotationX / 360 * TOTAL_POINT_ROW;
	else
		num = TOTAL_POINT_ROW - save_cylinderRotationX / 360 * TOTAL_POINT_ROW;

	currentEditingCurveIndex = NumberUtil.loopIndex(num, TOTAL_POINT_ROW);
	//trace(cylinderRotationPerFrame,currentEditingCurveIndex,num);
	updateInteractivePtPos(num);

	updateCurrentSoundCurve();
	updateSoundByte();

	curveSurface.suppressEventProcessing = false;
}

private function updateInteractivePtPos(num:Number):void {
	var pt:InteractivePoint;
	var percent:Percent = new Percent(num - Math.floor(num));
	for (var i:uint = 0; i < POINT_ROW_LENGTH; ++i) {
		pt = interactivePts[i];
		if (pt.color == IPOINT_DEFAULT_COLOR && !draggingPt) {
			num = NumberUtil.interpolate(percent, paths[currentEditingCurveIndex].segmentCollection.getItemAt(i)['y'], paths[NumberUtil.loopIndex(currentEditingCurveIndex + 1, TOTAL_POINT_ROW)].segmentCollection.getItemAt(i)['y']);
			pt.y = num;
		} else {
			if (pt.color == IPOINT_DOWN_COLOR)
				pt.y = NumberUtil.constrain(pt.y + 2, pt.restrict.top, pt.restrict.bottom);
			if (pt.color == IPOINT_UP_COLOR)
				pt.y = NumberUtil.constrain(pt.y - 2, pt.restrict.top, pt.restrict.bottom);
			var path:Path = paths[currentEditingCurveIndex];
			var cb:CubicBezierTo, seg:ISegment;
			pt = interactivePts[i];
			seg = path.segmentCollection.getItemAt(i);
			num = NumberUtil.interpolate(percent, pt.y, seg["y"]);
			seg["y"] = num;
			if (seg is CubicBezierTo) {
				cb = seg as CubicBezierTo;
				cb.cy1 = num;
			}
			seg = path.segmentCollection.getItemAt(i + 1);
			if (seg is CubicBezierTo) {
				cb = seg as CubicBezierTo;
				cb.cy = num;
			}
		}
	}
}

private function updateSoundByte():void {
	var sectionLength:uint = tempSoundBytes.length / (POINT_ROW_LENGTH - 1) * 0.125;
	var cb:CubicBezierTo, val:Number, valAry:Array; //var dump:String = "";
	tempSoundBytes.position = soundBytes.position = 0;
	for (i = 1; i < POINT_ROW_LENGTH; ++i) {
		cb = currentSoundCurve.segmentCollection.getItemAt(i) as CubicBezierTo;
		tempBezier.x0 = currentSoundCurve.segmentCollection.getItemAt(i - 1)['x'];
		tempBezier.y0 = currentSoundCurve.segmentCollection.getItemAt(i - 1)['y'];
		tempBezier.cx = cb.cx;
		tempBezier.cy = cb.cy;
		tempBezier.cx1 = cb.cx1;
		tempBezier.cy1 = cb.cy1;
		tempBezier.x1 = cb.x;
		tempBezier.y1 = cb.y;
		for (j = 0; j < sectionLength; ++j) {
			valAry = tempBezier.yAtX(tempBezier.x0 + j / sectionLength * (tempBezier.x1 - tempBezier.x0));
			if (valAry.length > 0) {
				val = NumberUtil.map(-valAry[0].y - CYLINDER_RADIUS, POINT_MIN_HEIGHT, POINT_MAX_HEIGHT, -1, 1);
			}
			tempSoundBytes.writeFloat(val);
			tempSoundBytes.writeFloat(val); //dump += " " + val.toFixed(3);
		}
	}
	//trace(dump)
	soundBytes.writeBytes(tempSoundBytes, 0, tempSoundBytes.length);
}

private function updateHandleSize():void {
	var cb:CubicBezierTo;
	for (i = 0; i < TOTAL_POINT_ROW; ++i) {
		for (j = 1; j < POINT_ROW_LENGTH; ++j) {
			cb = (paths[i] as Path).segmentCollection.getItemAt(j) as CubicBezierTo;
			cb.cx = (paths[i] as Path).segmentCollection.getItemAt(j - 1)['x'] + crvHandleSize;
			cb.cx1 = cb.x - crvHandleSize;
		}
	}
}

private function setVolume(volume:Number):void {
	var transform:SoundTransform = soundChannel.soundTransform;
	transform.volume = volume;
	soundChannel.soundTransform = transform;
}